function [k] = parallelCA(N,SamplingType,percent)
% Lorenzo-Seva, U. (2011). Horn's Parallel Analysis for Selecting the Number of Dimensions 
% in Correspondence Analysis. Methodology: European Journal of Research Methods for the 
% Behavioral and Social Sciences, 7 (3), 96-102.

%INPUT
%
% N                     THE EMPIRICAL CONTIGENCE TABLE
%
% SamplingType          1: Uniform random using indicator matrix
%                       2: Bootstrap sampling
%
% percent               CUT-OFF
%
% OUTPUT
%
% k      NUMBER OF ADVISED DIMENSIONS
%

k=0;

Ndatsets = 500;

%Size or N
[I,J]=size(N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% CA
Ir = ones(I,1);
Ic = ones(J,1);
n=Ir'*N*Ic;
P = (1/n) * N;
r  = P * Ic ;
Dr = diag(r);
c  = (Ir' * P)';
Dc = diag(c);
RP = Dr^(-1) * P;
CP = P * Dc^(-1);
K = min(I-1,J-1);
A =  Dr^(-1/2) * (P - r * c') * Dc^(-1/2);
chis = Ir'* (A.*A) * Ic * n;
ti = chis/n;

L = diag(svd(A));
realevals=diag(L*L);
K = min(I-1,J-1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% PA

if (SamplingType == 2),
    %RAW DATA
     X= [];
     for i=1:I,
         for j=1:J,
             for k=1:N(i,j),
                 X= [X; i j];
             end;
         end;
     end;
end;

tstart = tic;
bar1 = waitbar(0,'Parallel Analysis');
for a = 1 : Ndatsets;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% RANDOMDATASET
    if (SamplingType == 1),
        Nrand = RBurt(n,[I J],2);
    else
        Nrand = BootstrapC(X,size(N),2);
    end;
    Prand = (1/n) * Nrand;
    rrand  = Prand * Ic ;
    Drrand = diag(rrand);
    crand  = (Ir' * Prand)';
    Dcrand = diag(crand);
    Arand =  Drrand^(-1/2) * (Prand - rrand * crand') * Dcrand^(-1/2);
    chisrand = Ir'* (Arand.*Arand) * Ic * n;
    tirand = chisrand/n;
    Lrand = diag(diag(svd(Arand)));
    evals(:,a) = (Lrand.*Lrand)/tirand*ti;

    compT = toc(tstart);
    compT = compT*Ndatsets/a - compT;
    compH = floor(compT/(60*60));
    compT2 = compT - compH*60*60;
    compM = floor((compT2)/60);
    compS = round(compT2 - compM*60);
    if compH > 0,
        buff=sprintf('%.0f:%.0f:%.0f',compH, compM,compS);    
    elseif compM > 10,
        buff=sprintf('00:%.0f:%.0f',compM,compS);    
    elseif compM > 1,
        buff=sprintf('00:0%.0f:%.0f',compM,compS);    
    else,
        buff=sprintf('%.0f seconds',compS);    
    end;
    if compS>1,
       s_tmp=['Parallel Analysis (Remaining time = ' buff ')'];
       waitbar(a/Ndatsets,bar1,s_tmp);
    elseif compS==1,
       s_tmp=['Parallel Analysis (Remaining time = 1 second)'];
       waitbar(a/Ndatsets,bar1,s_tmp);
    else,
       s_tmp=['Parallel Analysis (already finishing)'];
       waitbar(a/Ndatsets,bar1,s_tmp);
    end;
    
    
end;
close(bar1);

means = (mean(evals,2));   % mean eigenvalues for each position.
evals = sort(evals,2);     % sorting the eigenvalues for each position.
percentiles = (evals(:,round((percent*Ndatsets)/100)));  % percentiles.

[m,tmp]=size(realevals);

for root = 1:m,
    if realevals(root,1) < means(root,1),
       nfPARPCA = root - 1;
       break;
   end;
end;
	
clear evals

disp(' ');
if (SamplingType == 1),
    disp( 'Ramdom samples procedure: Monte Carlo techniques');
else
    disp( 'Ramdom samples procedure: Bootstrap samples');
end;
disp(['Number of random samples: ' num2str(Ndatsets) ]);
disp(['Percentile computed     : ' num2str(percent) ]); disp([' '])
disp(' ');
disp(' ');
disp([' Principal inertia   Real data   Random data' ])
disp([' -----------------   ---------   -------------------'])
disp(['                                 Mean     Percentile'])
disp(' ');
for i=1:m,
    buff=sprintf('      %3.0f            %6.4f      %6.4f   %6.4f',i,realevals(i),means(i),percentiles(i));
    disp(buff)
end;
disp([' -----------------   ---------   -------------------'])
disp(' ');
disp(' ');
disp(['Number of dimensions advised for this dataset:  '  num2str(nfPARPCA) ]);
disp(' ');
disp(' Lorenzo-Seva, U. (2011). Horns Parallel Analysis for Selecting the Number of Dimensions');
disp(' in Correspondence Analysis. Methodology: European Journal of Research Methods for the'); 
disp(' Behavioral and Social Sciences, 7 (3), 96-102.');
disp(' ');

k = nfPARPCA;

return;
